// PACKAGES USED
const { Command } = require('@oclif/core');
const fs = require('fs');
const inquirer = require('inquirer');
const path = require('path');

// UTILITIES
const { initLogger } = require(path.join(__dirname, '../../utils/logger.js'));
const logger = initLogger('config');
const { fileChecker } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let prevConfig = {};
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json"))
  prevConfig = require("../../../config.json");

// ALL THE SERVICES BEING USED IN THE APPLICATION WITH THEIR NAMES AS KEYS
let defaultServices = {
  nodeconsumer: "Node Consumer",
  nrestserver: "Nrest Server",
  //iwfetl: "ETL",
  jobnodered: "Job Node Red",
  ncenter: "Ncenter",
  // reporting: "Reporting Server",
  // cred_auth: "CRED AUTH",
  // fixInterface2: "Fix Interface 4.2",
  // fixInterface4: "Fix Interface 4.4",
  // mailingEngine: "Mailing Engine",
};

let processFunctions = {
  fixInterface2: (fixInterfacePath) => {
    let filePath = path.join(
      fixInterfacePath,
      "/FIXTrading/config/credFix.cfg"
    );
    if (fileChecker([filePath])) {
      fs.readFile(
        filePath,
        { encoding: "utf-8", flag: "r" },
        async (err, data) => {
          if (err) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(err.message);
          } else {
            if (data.search(/scriptFile(.*)/g) !== -1)
              data = data.replace(
                /scriptFile(.*)/g,
                "scriptFile=" + fixInterfacePath
              );
            else data = data + "scriptFile=" + fixInterfacePath;
            if (data.search(/jsonFile(.*)/g) !== -1)
              data = data.replace(
                /jsonFile(.*)/g,
                "jsonFile=" + fixInterfacePath
              );
            else data = data + "jsonFile=" + fixInterfacePath;
            fs.writeFile(filePath, data, (err) => {
              if (err) {
                logger.error(
                  `An error occurred while writing changes in ${filePath}`
                );
                logger.error(err.message);
              }
            });
          }
        }
      );
    } else
      logger.error("Fix Interface 4.2 file does not exist. Run config:init");
  },
  fixInterface4: (fixInterfacePath) => {
    let filePath = path.join(
      fixInterfacePath,
      "/FIXTrading/config/credFix.cfg"
    );
    if (fileChecker([filePath])) {
      fs.readFile(
        filePath,
        { encoding: "utf-8", flag: "r" },
        async (err, data) => {
          if (err) {
            logger.error(`An error occurred while reading ${filePath}`);
            logger.error(err.message);
          } else {
            if (data.search(/scriptFile(.*)/g) !== -1)
              data = data.replace(
                /scriptFile(.*)/g,
                "scriptFile=" + fixInterfacePath
              );
            else data = data + "scriptFile=" + fixInterfacePath;
            if (data.search(/jsonFile(.*)/g) !== -1)
              data = data.replace(
                /jsonFile(.*)/g,
                "jsonFile=" + fixInterfacePath
              );
            else data = data + "jsonFile=" + fixInterfacePath;
            fs.writeFile(filePath, data, (err) => {
              if (err) {
                logger.error(
                  `An error occurred while writing changes in ${filePath}`
                );
                logger.error(err.message);
              }
            });
          }
        }
      );
    } else
      logger.error("Fix Interface 4.4 file does not exist. Run config:init");
  },
};

// MAIN COMMAND LINE FUNCTION
class InitCommand extends Command {
  async run() {
    // MULTISELECT OPTION WHICH ALLOWS USER TO CHOOSE WHICH SERVICES TO INITIALIZE
    let { services } = await inquirer.prompt({
      name: "services",
      type: "checkbox",
      message: "Select the Services Installed:",
      choices: Object.values(defaultServices), // SETTING DISPLAY CHOICES TO USER FOR THE MULTISELECT
      default: Object.keys(prevConfig).map(
        // SETTING CHECKED SERVICES FOR THE SERVICES THAT HAVE BEEN INITIALIZED INITIALLY
        (service) => defaultServices[service]
      ),
      filter: (res) =>
        res.map((r) =>
          Object.keys(defaultServices).find((key) => defaultServices[key] === r)
        ),
      // FILTERING THE RESULT TO CONVERT THEM FROM SERVICE DISPLAY NAME TO SERVICE KEY CODE
    });
    let inputs = {
      // DEFINED AN OBJECT WITH OBJECTS FOR ALL THE INPUT PROMPTS FOR DIFFERENT SERVICES
      warfile: {
        name: "warfile",
        default: prevConfig["warfile"],
        message: "Enter WAR folder path",
        validate: (res) =>
          fileChecker(
            [res, path.join(res, "/Framewrk/common/iconfig.jds")],
            "returnMessage"
          ),
        filter: (res) => res.split("\\").join("/"),
      },
      apps: {
        name: "apps",
        default: prevConfig["apps"],
        message: 'Enter Apps folder path',
        validate: (res) => fileChecker(
          [
            res,
            path.join(res, 'app-builder/dist/config.js'),
            // path.join(res, 'appcomponents/dist/config.js'),
            path.join(res, 'Custom_dashboard/dist/config.js'),
            path.join(res, 'Historical/dist/config.js'),
            path.join(res, 'Exception/dist/config.js'),
            path.join(res, 'NPA/dist/config.js'),
            // path.join(res, 'Position_Dashboard/dist/config.js'),
            path.join(res, 'Returns_dashboard/dist/config.js'),
            path.join(res, 'Risk Management/dist/config.js'),
            path.join(res, 'Trader Desk/dist/config.js'),
            path.join(res, 'FIS/dist/config.js'),
            path.join(res, 'Operations/dist/config.js'),
          ],
          "returnMessage"
        ),
        filter: (res) => res.split("\\").join("/"),
      },
      nodeconsumer: {
        name: "nodeconsumer",
        default: prevConfig["nodeconsumer"],
        message: "Enter node consumer microservice path",
        validate: (res) =>
          fileChecker(
            [res, path.join(res, "/Handlers/config.json")],
            "returnMessage"
          ),
        filter: (res) => res.split("\\").join("/"),
      },
      nrestserver: {
        name: "nrestserver",
        default: prevConfig["nrestserver"],
        message: "Enter node restserver microservice path",
        validate: (res) =>
          fileChecker(
            [res, path.join(res, "/fundsnservices/dbconfig.json")],
            "returnMessage"
          ),
        filter: (res) => res.split("\\").join("/"),
      },
      // iwfetl: {
      //   name: "iwfetl",
      //   default: prevConfig["iwfetl"],
      //   message: "Enter ETL microservice path",
      //   validate: (res) =>
      //     fileChecker(
      //       [res, path.join(res, "/custom/config.json")],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      jobnodered: {
        name: "jobnodered",
        default: prevConfig["jobnodered"],
        message: "Enter jobnodered microservice path",
        validate: (res) =>
          fileChecker([res, path.join(res, "/config.json")], "returnMessage"),
        filter: (res) => res.split("\\").join("/"),
      },
      ncenter: {
        name: "ncenter",
        default: prevConfig["ncenter"],
        message: "Enter notification center microservice path",
        validate: (res) =>
          fileChecker(
            [res, path.join(res, "/dist/server/.env")],
            "returnMessage"
          ),
        filter: (res) => res.split("\\").join("/"),
      },
      // reporting: {
      //   name: "reporting",
      //   default: prevConfig["reporting"],
      //   message: "Enter reporting server microservice path",
      //   validate: (res) =>
      //     fileChecker(
      //       [res, path.join(res, "/app/config/.env")],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      // cred_auth: {
      //   name: "cred_auth",
      //   default: prevConfig["cred_auth"],
      //   message: "Enter credauth microservice path",
      //   validate: (res) =>
      //     fileChecker(
      //       [res, path.join(res, "/modules/config.json")],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      // fixInterface2: {
      //   name: "fixInterface2",
      //   default: prevConfig["fixInterface2"],
      //   message: "Enter fix interface 4.2 path (EQ)",
      //   validate: (res) =>
      //     fileChecker(
      //       [res, path.join(res, "/FIXTrading/config/credFix.cfg")],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      // fixInterface4: {
      //   name: "fixInterface4",
      //   default: prevConfig["fixInterface4"],
      //   message: "Enter fix interface 4.4 path (Debt)",
      //   validate: (res) =>
      //     fileChecker(
      //       [res, path.join(res, "/FIXTrading/config/credFix.cfg")],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      // mailingEngine: {
      //   name: "mailingEngine",
      //   default: prevConfig["mailingEngine"],
      //   message: "Enter mailing engine path",
      //   validate: (res) =>
      //     fileChecker(
      //       [
      //         res,
      //         path.join(
      //           res,
      //           `OutboxMail_1.${process.platform == "win32" ? "bat" : "sh"}`
      //         ),
      //         path.join(
      //           res,
      //           `Scheduler_1.${process.platform == "win32" ? "bat" : "sh"}`
      //         ),
      //       ],
      //       "returnMessage"
      //     ),
      //   filter: (res) => res.split("\\").join("/"),
      // },
      ngnix: {
        name: "ngnix",
        default: prevConfig["ngnix"],
        message: 'Enter nginx configuration file path',
        validate: (res) => fileChecker([res], "returnMessage"),
        filter: (res) => res.split("\\").join("/"),
      }
    };
    // PROMPTING THE USER FOR THE PATHS OF ALL THE SELECTED SERVICES AND WARFILE AND NGNIX AS DEFAULT
    let config = await inquirer.prompt([
      inputs.warfile,
      inputs.ngnix,
      inputs.apps,
      ...services.map((service) => inputs[service]),
    ]);
    // EXTRACTING APPSERVER FROM WARFILE PATH
    config.appserver = config.warfile.slice(
      0,
      config.warfile.indexOf("standalone")
    );
    config = { ...prevConfig, ...config }; // COMBINING PREVCONFIG AND NEW CONFIG
    // CHECKING IF ANY SERVICES WERE SELECTED AND IF YES CREATING THE UPLOAD AND DOWNLOAD FOLDER FROM THE EXTRACTED PATH OF A SERVICE
    if (services.length > 0) {
      let uploadFolder = path.join(config[services[0]], "../upload"),
        downloadFolder = path.join(config[services[0]], "../download");
      try {
        if (!fs.existsSync(uploadFolder)) {
          fs.mkdirSync(uploadFolder);
          logger.info(`Upload folder created at ${uploadFolder}`);
        }
        if (!fs.existsSync(downloadFolder)) {
          fs.mkdirSync(downloadFolder);
          logger.info(`Download folder created at ${downloadFolder}`);
        }
      } catch (err) {
        logger.error(
          `An error occurred while creating ${uploadFolder} and ${downloadFolder}`
        );
        logger.error(err.message);
      }
    }
    // CHECKING IF THE FIX INTERFACE 4.2 SERVICE IS INITIALIZED AND UPDATING ITS PATH IN THE FILE
    if (config["fixInterface2"])
      processFunctions.fixInterface2(config["fixInterface2"]);
    // CHECKING IF THE FIX INTERFACE 4.4 SERVICE IS INITIALIZED AND UPDATING ITS PATH IN THE FILE
    if (config["fixInterface4"])
      processFunctions.fixInterface4(config["fixInterface4"]);
    // WRITING THE CHANGES INTO THE CONFIG FILE
    fs.writeFile(
      __dirname.slice(0, __dirname.indexOf("src")) + "config.json",
      JSON.stringify(config, null, 2),
      () => {
        logger.info("fundscli Initialisation Successful");
      }
    );
  }
}

InitCommand.description = `Initialize the application. Takes input of paths of the JBOSS server directory and all the other microservice and creates a config.json file and stores the paths in it.
Also asks the user if Oracle Instant Client is to be installed or not and installs it directly`;
module.exports = InitCommand;
