// PACKAGES USED
const { Command } = require('@oclif/core');
const fs = require('fs');
const path = require('path');
const inquirer = require('inquirer');
const oracledb = require('oracledb');
oracledb.autoCommit = true;

// UTILITIES
const logger = require("../../utils/logger");
const { fileChecker, serviceFiles } = require("../../utils/fileChecker");

// CHECKING IF CONFIG FILE EXISTS AND REQUIRING THE DATA IN IT
let config, encrypt;
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf("src")) + "config.json"))
  config = require("../../../config.json");
let filePaths = {
  iwfetl: "/custom/config.json",
  jobnodered: "/config.json",
  nodeconsumer: "/Handlers/config.json",
  nrestserver: "/fundsnservices/dbconfig.json",
};

// CREATED AN OBJECT THAT HAS KEYS AS KEYWORD NAMES OF SERVICES AND A FUNCTION THAT MAKES CHANGES TO THE PARTICULAR SERVICE FILES
const processFunctions = {
  // FUNCTION TO MAKE CHANGES IWFETL config.json FILES
  iwfetl: ({ globalDateFormat }) => {
    let filePath = path.join(config.iwfetl, filePaths.iwfetl);
    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.database) data.database = {};
        data.database.DateFormat = globalDateFormat;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else logger.error("ETL file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO JOBNODERED config.json FILE
  jobnodered: ({ globalDateFormat, globalDateTimeFormat }) => {
    let filePath = path.join(config.jobnodered, filePaths.jobnodered);
    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        if (!data.databases) data.databases = {};

        if (!data.databases.ideal) data.databases.ideal = {};
        data.databases.ideal["DATE_FORMAT"] = globalDateFormat;
        data.databases.ideal["DATE_TIME_FORMAT"] = globalDateTimeFormat;
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else
        logger.error(
          "job-center-node-red file does not exist. Run config:init"
        );
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES NODECONSUMER config.json FILE
  nodeconsumer: ({ globalDateFormat, globalDateTimeFormat, nginx_url }) => {
    let filePath = path.join(config.nodeconsumer, filePaths.nodeconsumer);
    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        data["DateFormat"] = globalDateFormat;
        data["DATE_TIME_FORMAT"] = globalDateTimeFormat;
        if (!data.notification) data.notification = {};
        data.notification["url"] = new URL("/ncenter/", nginx_url);
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else
        logger.error("funds-node-consumer file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO NRESTSERVER dbconfig.json FILE
  nrestserver: ({
    globalDateFormat,
    globalDateTimeFormat,
    metabase,
    metabaseSecretKey,
    nginx_url,
  }) => {
    let filePath = path.join(config.nrestserver, filePaths.nrestserver);

    try {
      if (fileChecker([filePath])) {
        const data = require(filePath);
        data["DATE_FORMAT"] = globalDateFormat;
        data["DATE_TIME_FORMAT"] = globalDateTimeFormat;
        data["METABASE"] = metabase;
        data["METABASE_SECRET_KEY"] = metabaseSecretKey;
        data["NGINX_IP"] = nginx_url;
        let microservicesPath = config.nrestserver.slice(
          0,
          config.nrestserver.indexOf("nrestserver")
        );
        data["UPLOAD_PATH"] = microservicesPath + "upload/";
        data["DOWNLOAD_PATH"] = microservicesPath + "download/";
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
          if (err) {
            logger.error(
              `An error occurred while writing changes in ${filePath}`
            );
            logger.error(err.message);
          }
        });
      } else logger.error("nrest-server file does not exist. Run config:init");
    } catch (e) {
      logger.error(`An error occurred while reading ${filePath}`);
      logger.error(e.message);
    }
  },
  // FUNCTION TO MAKE CHANGES TO THE DATABASE
  editDatabase: async (
    { globalDateFormat, globalDateTimeFormat },
    database
  ) => {
    let connection;
    try {
      connection = await oracledb.getConnection(database);
      await connection.execute(`BEGIN UPDATE parameter SET PARAMVALUE = :1 WHERE PARAMNAME = 'oracleShortDateformat'; UPDATE parameter SET PARAMVALUE = :2 WHERE PARAMNAME = 'oracleLongDateformat'; END;`, [globalDateFormat, globalDateTimeFormat]);
    } catch (err) {
      logger.error("An error occurred while writing changes in oracledb");
      logger.error(err.message);
    } finally {
      if (connection) {
        try {
          await connection.close();
        } catch (err) {
          logger.error("An error occurred while writing changes in oracledb");
          logger.error(err.message);
        }
      }
    }
  },
  // FUNCTION TO MAKE CHANGES TO WARFILES
  warfiles: ({
    applicationtitle,
    client_image,
    client_name,
    client_address,
    globalDateFormat,
    globalTimeFormat,
    globalDateTimeFormat,
    appsglobalTimeFormat,
    appsglobalDateFormat,
    appsglobalDateTimeFormat,
    nginx_url,
  }) => {
    let instructions = [
      {
        filePath: path.join(config.warfile, "/Framewrk/common/iconfig.jds"),
        update: [
          { expression: "iwebz.applicationtitle", value: applicationtitle },
          { expression: "iwebz.client_image", value: client_image },
          { expression: "iwebz.client_name", value: client_name },
          { expression: "iwebz.client_address", value: client_address },
          {
            expression: "iwebz.globalDateFormat",
            value:
              globalDateFormat === "DD-MM-YYYY" ? "dd-MM-yyyy" : "MM-dd-yyyy",
          },
          { expression: "iwebz.globalTimeFormat", value: globalTimeFormat },
          {
            expression: "iwebz.globalDateTimeFormat",
            value:
              (globalDateFormat === "DD-MM-YYYY"
                ? "dd-MM-yyyy "
                : "MM-dd-yyyy ") + globalTimeFormat,
          },
          { expression: "iwebz.nginx_url", value: nginx_url },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/app-builder/dist/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: globalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: globalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: globalTimeFormat },
        ],
      },
      // {
      //   filePath: path.join(
      //     config.apps,
      //     "/appcomponents/dist/config.js"
      //   ),
      //   update: [
      //     { expression: "config.globalDateFormat", value: globalDateFormat },
      //     {
      //       expression: "config.globalDateTimeFormat",
      //       value: globalDateTimeFormat,
      //     },
      //     { expression: "config.globalTimeFormat", value: globalTimeFormat },
      //   ],
      // },
      {
        filePath: path.join(
          config.apps,
          "/Custom_dashboard/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
      },
      {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/FIS/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Exception/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Historical/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/NPA/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Operations/dist/js/config.js"
        ),
        update: [
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(config.apps, "/Position_Dashboard/dist/js/config.js"),
        update:[
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Returns_dashboard/dist/js/config.js"
        ),
        update:[
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Risk Management/dist/js/config.js"
        ),
        update:[
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      },
      {
        filePath: path.join(
          config.apps,
          "/Trader Desk/dist/js/config.js"
        ),
        update:[
          {
            expression: "config.globalDateFormatShort",
            value: appsglobalDateFormat,
          },
          {
            expression: "config.globalDateFormatLong",
            value: appsglobalDateTimeFormat,
          },
          { expression: "config.globalTimeFormat", value: appsglobalTimeFormat },
        ],
      }
    ];
    if (fileChecker(instructions))
      instructions.map(({ filePath, update }) => {
        fs.readFile(
          filePath,
          { encoding: "utf-8", flag: "r" },
          async (err, data) => {
            if (err) {
              logger.error(`An error occurred while reading ${filePath}`);
              logger.error(err.message);
            }
            update.map(({ expression, value }) => {
              if (data.search(new RegExp(`${expression}(.*)`, "g")) !== -1)
                data = data.replace(
                  new RegExp(`${expression}(.*)`, "g"),
                  `${expression} = "` + value + '";'
                );
              else data = data + `\n${expression} = \"` + value + '";';
            });
            fs.writeFile(filePath, data, (err) => {
              if (err) {
                logger.error(
                  `An error occurred while writing changes in ${filePath}`
                );
                logger.error(err.message);
              }
            });
          }
        );
      });
  },
};

// MAIN COMMAND FUNCTION
class ProductCommand extends Command {
  async run() {
    // CHECKING IF CONFIG FILES EXIST OR NOT
    if (config) {
      // CALLING FILE CHECKER FUNCTION WHICH CHECKS THAT ALL THE FILES BEING USED EXIST ARE READABLE AND WRITEABLE
      // THE SERVICE FILES FUNCTION GIVES BACK US AN ARRAY OF ABSOLUTE PATH FROM THE ROOT DIRECTORY FOR DIFFERENT SERVICE FILES
      let filesUsed = [
        ...serviceFiles(config, filePaths),
        path.join(config.warfile, "/Framewrk/common/iconfig.jds"),
        path.join(config.apps, "/FIS/dist/js/config.js"),
        path.join(config.apps, "/app-builder/dist/config.js"),
        // path.join(config.apps, "/appcomponents/dist/config.js"),
        path.join(config.apps, "/Custom_dashboard/dist/js/config.js"),
        path.join(config.apps, "/Historical/dist/js/config.js"),
        path.join(config.apps, "/Exception/dist/js/config.js"),
        path.join(
          config.apps,
          "/NPA/dist/js/config.js"
        ),
        path.join(config.apps, "/Operations/dist/js/config.js"),
        path.join(
          config.apps,
          "/Position_Dashboard/dist/js/config.js"
        ),
        path.join(config.apps, "/Returns_dashboard/dist/js/config.js"),
        path.join(
          config.apps,
          "/Risk Management/dist/js/config.js"
        ),
        path.join(
          config.apps,
          "/Trader Desk/dist/js/config.js"
        )
      ];
      if (fileChecker(filesUsed)) {
        let database,
          connection,
          flag = true;
        // DO WHILE LOOP TO CHECK IF DATABASE CREDENTIALS PROVIDED CONNECT US TO THE DATABASE
        do {
          try {
            // PROMPTING THE USER FOR DATABASE INPUTS
            database = await inquirer.prompt([
              {
                message: "Enter database username",
                name: "user",
                validate: (res) => {
                  if (res === "") return "Enter a value";
                  return true;
                },
              },
              {
                message: "Enter database password",
                type: "password",
                name: "password",
                validate: (res) => {
                  if (res === "") return "Enter a value";
                  return true;
                },
              },
              {
                message: "Enter database connection string",
                name: "connectionString",
                validate: (res) => {
                  if (res === "") return "Enter a value";
                  return true;
                },
              },
            ]);
            flag = false;
            connection = await oracledb.getConnection(database);
          } catch (error) {
            logger.error("Could not connect to database");
            flag = true;
          } finally {
            if (connection) {
              try {
                await connection.close();
              } catch (err) {
                logger.error(err);
              }
            }
          }
        } while (flag);
        let inputs = await inquirer.prompt([
          {
            message: "Enter application title",
            name: "applicationtitle",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter application logo path w.r.t WAR",
            name: "client_image",
            validate: (res) =>
              fileChecker([path.join(config.warfile, res)], "returnMessage"),
          },
          {
            message: "Enter client name",
            name: "client_name",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter client address",
            name: "client_address",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            type: "list",
            message: "Select dateformat",
            choices: ["DD-MM-YYYY", "MM-DD-YYYY"],
            name: "globalDateFormat",
          },
          {
            type: "list",
            message: "Select timeformat",
            choices: ["hh24:mi:ss", "hh12:mi:ss"],
            name: "globalTimeFormat",
          },
          {
            type: "list",
            message: "Select dateformat for war apps config",
            choices: ["dd/mm/yyyy", "mm/dd/yyyy"],
            name: "appsglobalDateFormat",
          },
          {
            type: "list",
            message: "Select timeformat for war apps config",
            choices: ["hh:mm:ss", "HH:mm:ss"],
            name: "appsglobalTimeFormat",
          },
          {
            type: "list",
            message: "Select datetimeformat for war apps config",
            choices: ["dd/mm/yyyy:HH:mm:ss", "dd/mm/yyyy:hh:mm:ss"],
            name: "appsglobalDateTimeFormat",
          },
          {
            message: "Enter application url",
            name: "nginx_url",
            validate: (res) => {
              if (res === "") return "Enter a value";
              try {
                new URL("/ncenter/", res);
                return true;
              } catch (error) {
                return error.message;
              }
            },
          },
          {
            message: "Enter metabase username",
            name: "metabase[USERNAME]",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
          },
          {
            message: "Enter metabase password",
            name: "metabase[PASSWORD]",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
            type: "password",
          },
          {
            message: "Enter metabase secret key",
            name: "metabaseSecretKey",
            validate: (res) => {
              if (res === "") return "Enter a value";
              return true;
            },
            type: "password",
          },
          {
            message: "Would you like to encrypt sensitive data?",
            name: "choice_encrypt",
            type: "list",
            choices: ["Yes", "No"],
          },
        ]);
        inputs["globalDateTimeFormat"] =
          inputs["globalDateFormat"] + " " + inputs["globalTimeFormat"];

        if (inputs.choice_encrypt === "Yes") {
          encrypt = require("../../utils/encrypt")(
            path.join(__dirname, "../../../", "keys/public.pem")
          );
          if (inputs.metabase && inputs.metabase["PASSWORD"]) {
            inputs.metabase["PASSWORD"] = encrypt(inputs.metabase["PASSWORD"]);
          }
        }

        // MAPPING OVER THE CONFIG OBJECT TO CALL CHANGES ON ONLY THOSE SERVICES THAT HAVE BEEN INITIALIZED AND THEIR PATHS ARE PRESENT IN config.json
        Object.keys(config).forEach((service, i) => {
          if (processFunctions[service]) processFunctions[service](inputs);
        });
        processFunctions["editDatabase"](inputs, database);
        processFunctions["warfiles"](inputs);
        if (!config.apps) logger.warn(`As Nginx web server is separate, date format changes in apps is to be done manually.`)
        logger.info("Product Initialisation Successful");

        return { status:"success" }
      }
    } else logger.error("Config file not found. Run config:init command");
  }
}

ProductCommand.description = `Change the product details`;
module.exports = ProductCommand;
