const { Command, Flags } = require('@oclif/core');
const fs = require('fs');
const path = require('path');

const { logger } = require("../../utils/logger");
const NodeRSA = require("node-rsa");

let config = {};
if (fs.existsSync(__dirname.slice(0, __dirname.indexOf('src')) + 'config.json')) config = require('../../../config.json');

function flagsValidator(flags) {
    let pathExist = fs.statSync(flags.location).isDirectory();
    if (flags.bits !== '1024' && flags.bits !== '2048') {
        throw new Error(`Bad option: --bits=${flags.bits}. Please run 'fundscli generate:keys --help' command for help`);
    } else if (!pathExist) {
        throw new Error(`Bad option: --location=${flags.path}. Please run "fundscli generate:keys --help' command for help`);
    } else if (flags['pubkey-format'] !== 'pkcs1' && flags['pubkey-format'] !== 'pkcs8') {
        throw new Error(`Bad option: --pubkey-format=${flags['pubkey-format']}. Please run 'fundscli generate:keys --help' command for help`);
    } else if (flags['pvtkey-format'] !== 'pkcs1' && flags['pvtkey-format'] !== 'pkcs8') {
        throw new Error(`Bad option: --pvtkey-format=${flags['pvtkey-format']}. Please run 'fundscli generate:keys --help' command for help`);
    }
}
class GenerateKeys extends Command {
    async run() {
        try {
            let Key, keysFolder, pubKey, pvtKey;
            const { flags } = await this.parse(GenerateKeys);

            if (Object.keys(flags).length === 0) {
                Key = new NodeRSA().generateKeyPair();
                keysFolder = path.join(config.nrestserver, '../keys');
                pubKey = 'pkcs8-public-pem';
                pvtKey = 'pkcs1-private-pem';
            } else {
                if (!flags.bits || !flags.location || !flags['pubkey-format'] || !flags['pvtkey-format']) {
                    throw new Error('Make sure you have entered all flags properly');
                } else {
                    flagsValidator(flags);
                    flags.bits = Number(flags.bits);
                    Key = new NodeRSA(flags.bits).generateKeyPair(flags.bits);
                    keysFolder = flags.location;
                    pubKey = `${flags['pubkey-format']}-public-pem`;
                    pvtKey = `${flags['pvtkey-format']}-private-pem`;
                }
            }

            // `microservice/nrest` then keys folder will be `microservice`

            if (fs.existsSync(keysFolder) && fs.existsSync(path.join(keysFolder, 'private.pem')) && fs.existsSync(path.join(keysFolder, 'public.pem'))) {
                logger.info(`Encryption keys are already present in '${keysFolder}' location. Delete existing encryption key files and try again.`);
                return { status: 'success' };
            }
            if (!fs.existsSync(keysFolder)) {
                fs.mkdirSync(keysFolder, { recursive: true });
                logger.info(`Keys folder '${keysFolder}' created`);
            }

            const [privateKeyData, publicKeyData] = [Key.exportKey(pvtKey), Key.exportKey(pubKey)];
            if (Object.keys(flags).length === 0) {
                fs.writeFileSync(path.join(__dirname, '../../../', 'keys/private.pem'), privateKeyData);
                fs.writeFileSync(path.join(__dirname, '../../../', 'keys/public.pem'), publicKeyData);
                fs.writeFileSync(path.join(keysFolder, 'private.pem'), privateKeyData);
                fs.writeFileSync(path.join(keysFolder, 'public.pem'), publicKeyData);
                logger.debug(`private key ${path.join(__dirname, '../../../', 'keys/private.pem')} generated`);
                logger.debug(`public key ${path.join(__dirname, '../../../', 'keys/public.pem')} generated`);
            } else {
                fs.writeFileSync(path.join(keysFolder, 'private.pem'), privateKeyData);
                fs.writeFileSync(path.join(keysFolder, 'public.pem'), publicKeyData);
                logger.debug(`private key ${keysFolder}/private.pem generated`);
                logger.debug(`public key ${keysFolder}/public.pem generated`);
            }

            logger.info(`private key 'private.pem' generated`);
            logger.info(`public key 'public.pem' generated`);
            logger.info(`private key 'private.pem' moved to '${keysFolder}'`);
            logger.info(`public key 'public.pem' moved to '${keysFolder}'`);
        } catch (error) {
            logger.error(`Can not generate encryption key files! More details below`);
            logger.error(error.stack);
        }
        return { status: 'success' };
    }
}
GenerateKeys.flags = {
    bits: Flags.string({ description: 'The size of the encryption key to generate in bits. It should be 1024 OR 2048. e.g --bits=1024' }),
    location: Flags.string({ description: 'Path where encryption keys should be generated. e.g= --location="/credence/microservice/"' }),
    'pubkey-format': Flags.string({ description: 'Public Key format pkcs8 or pkcs1 e.g --pubkey-format=pkcs8' }),
    'pvtkey-format': Flags.string({ description: 'Private key format pkcs8 or pkcs1 e.g --pvtkey-format=pkcs8' }),
};

GenerateKeys.description = `Generate private and public encryption keys`;
module.exports = GenerateKeys;
