const path = require('path');
const os = require('os');
const fs = require('fs');
const { prompt } = require('enquirer');
const Listr = require('listr');

const readPromise = require('util').promisify(fs.readdir);

const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { microservicesInstallation, microservicesUninstall } = require(path.join(__dirname, '../../utils/helper/mservices-helper.js'));
const { appServerInput, appServerUninstall } = require(path.join(__dirname, './app-server.js'));

const logger = initLogger('product');

async function microserviceInput(directory, all) {
  try {
    let microInp;
    let microservicesToInstall = { microservices: [] };

    if (all === 'no') {
      microInp = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Would you like to install any services ? ',
          choices: ['yes', 'no'],
        }
      ]);
    }

    if (all === 'yes' || microInp.answer === 'yes') {
      const microserviceDependencies = await prompt([
        {
          type: 'select',
          name: 'all',
          message: 'Would you like to install all the services ? ',
          choices: ['yes', 'no'],
        }
      ]);
      if (microserviceDependencies.all === 'no') {
        const files = fs
          .readdirSync(path.resolve(`${directory}/Microservices`))
          .filter((el) => el.includes('.zip'))
          .map((el) => el.replace('.zip', ''));

        microservicesToInstall = await prompt([
          {
            type: 'multiselect',
            name: 'microservices',
            message: 'Select the microservices to install',
            choices: files,
            initial: files,
            instructions: true,
          }
        ])
      }
      const appServerChoice = await appServerInput(all);
      return { microserviceDependencies, microservicesToInstall: microservicesToInstall.microservices, microInp, appServerChoice, };
    }
    return { microInp };
  } catch (error) {
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during taking microservice input : ${error.stack}`)

  }
}

async function microservices(directory, dependencies, microservicesToInstall, appServerChoice) {
  try {
    const microservicesDir = path.resolve(`${directory}/Microservices`);

    if (!fs.existsSync(`${HOME_DIR}/credence/microservices`)) {
      fs.mkdirSync(`${HOME_DIR}/credence/microservices`)
    }

    function arrayEquals(a, b) {
      return Array.isArray(a) && Array.isArray(b) && a.length === b.length && a.every((val, index) => val === b[index]);
    }
    const zipFiles = fs
      .readdirSync(microservicesDir)
      .filter((fileName) => fileName.endsWith('.zip'))
      .map((fileName) => fileName.replace('.zip', ''));

    if (dependencies.all === 'yes') {
      try {
        const installedFiles = await readPromise(path.resolve(`${HOME_DIR}/credence/microservices`));
        const filteredInstalledFiles = installedFiles.filter((fileName) => !fileName.includes('.'));

        if (fs.existsSync(path.resolve(`${HOME_DIR}/credence/microservices`)) && arrayEquals(zipFiles, filteredInstalledFiles) && (fs.existsSync(`${path.resolve(`${HOME_DIR}/credence/JBOSS-7.1PEMI - Copy`)}`) || fs.existsSync(`${path.resolve(`${HOME_DIR}/credence/wildfly`)}`))) {
          logger.info('Microservices are already installed');
        } else {
          return await microservicesInstallation(directory, dependencies.all, [], appServerChoice);
        }
      } catch (e) {
        logger.error(e.stack);
      }
    } else {
      const selectedZipFiles = microservicesToInstall.map((el) => el + '.zip');
      return await microservicesInstallation(directory, dependencies.all, selectedZipFiles, appServerChoice);
    }
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during microservices : ${error.stack}`)

  }
}

function microservicesUninstallation(servicesDetails) {
  try {
    return new Listr([
      {
        title: 'Deleting Services',
        task: async () => await microservicesUninstall(servicesDetails),
      },
      {
        title: 'Deleting App Server',
        task: async () => await appServerUninstall(servicesDetails.applicationServer),
      }
    ]);
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during microservices uninstall : ${error.stack}`)
  }
}

module.exports = {
  microservices,
  microserviceInput,
  microservicesUninstallation,
};
