const fs = require('fs');
const path = require('path');
const execPromise = require('util').promisify(require('child_process').exec);
const os = require('os');
const { readConfig } = require('../utils');
const { initLogger, CliError, HOME_DIR } = require(path.join(__dirname, '../logger.js'));

const logger = initLogger('product');

async function pm2Redhat8_6Install(directory, folderName) {
  try {
    const config = readConfig();
    const pm2Result = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat8_6/scripts/install/pm2.sh ${directory} ${HOME_DIR}`)
    if (pm2Result.stdout.length > 0) logger.info(`stdout : ${pm2Result.stdout}`);
    if (pm2Result.stderr.length > 0) logger.info(`stderr : ${pm2Result.stderr}`);
    if (pm2Result.stdout.includes('5.1.0')) {
      config.dependencies.pm2.path = `${HOME_DIR}/credence/installation/node-v18.17.1-linux-x64/lib/node_modules/pm2`;
      config.dependencies.pm2.isInstalled = true;
      const newConfig = JSON.stringify(config, null, 2);
      fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
    } else logger.error(`pm2 was not installed properly `);
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}
async function pm2Redhat9Install(directory, folderName) {
  try {
    const config = readConfig();
    logger.info(`bash ${directory}/Microservices-Prerequisite/Linux/${folderName}/scripts/install/pm2.sh ${directory} ${HOME_DIR}`);
    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/install/pm2.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}
async function pm2Ubuntu20Install(directory, folderName) {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/install/pm2.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}


async function pm2Redhat8_6Uninstall() {
  try {
    const config = readConfig();
    const pm2Result = await execPromise(`bash ${config.init.path}/Microservices-Prerequisite/Linux/redhat8_6/scripts/uninstall/pm2.sh ${HOME_DIR}`)
    if (pm2Result.stdout.length > 0) logger.info(`stdout : ${pm2Result.stdout}`);
    if (pm2Result.stderr.length > 0) logger.info(`stderr : ${pm2Result.stderr}`);
    config.dependencies.pm2.path = ``;
    config.dependencies.pm2.isInstalled = false;
    const newConfig = JSON.stringify(config, null, 2);
    fs.writeFileSync(path.join(__dirname, '../../../product_cfg.json'), newConfig);
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}
async function pm2Redhat9Uninstall() {
  try {
    const config = readConfig();

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/redhat9/scripts/uninstall/pm2.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}
async function pm2Ubuntu20Uninstall() {
  try {

    const value = await execPromise(`bash ${directory}/Microservices-Prerequisite/Linux/ubuntu20/scripts/pm2.sh ${directory} ${HOME_DIR}`)
    if (value.stdout.length > 0) logger.info(`stdout : ${value.stdout}`)
    if (value.stderr.length > 0) logger.info(`stderr : ${value.stderr}`)
  } catch (err) {
    throw new CliError(`Execution of pm2.sh failed due to ${err.stack}`)
  }
}


async function pm2Installation(directory) {
  try {
    const config = readConfig()
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/installation`))) {
      fs.mkdirSync(path.resolve(`${HOME_DIR}/credence/installation`));
    }
    if (!fs.existsSync(path.resolve(`${HOME_DIR}/credence/microservices`))) {
      fs.mkdirSync(`${HOME_DIR}/credence/microservices`);
    }
    switch (true) {
			case config.os.version.includes('8'):
				await pm2Redhat8_6Install(directory, 'redhat8_6');
				break;
			case config.os.version.includes('9'):
				await pm2Redhat9Install(directory, 'redhat9');
				break;
			case config.os.version.includes('ubuntu'):
				await pm2Ubuntu20Install(directory, 'redhat8_6');
				break;
			default:
				logger.info('No matching case found.');
		}
  } catch (error) {
    logger.error(`Error in PM2 Installation due to : ${error}`);
    throw new CliError(`Error in PM2 Installation due to : ${error}`);
  }
}



async function pm2Uninstall(pm2Details) {
  try {
    if (pm2Details.isInstalled) {
      const config = readConfig()
      switch (true) {
        case config.os.version.includes('8'):
          await pm2Redhat8_6Uninstall();
          break;
        case config.os.version.includes('9'):
          await pm2Redhat9Uninstall();
          break;
        case config.os.version.includes('ubuntu'):
          await pm2Ubuntu20Uninstall();
          break;
      }
    } else {
      logger.info(`pm2 is not installed`);
    }
  } catch (error) {
    logger.error(`Error in pm2Uninstall due to : ${error}`);
    throw new CliError(`Error in pm2Uninstall due to : ${error}`);
  }
}
module.exports = { pm2Installation, pm2Uninstall };
