const path = require('path');
const Listr = require('listr');
const { prompt } = require('enquirer');
const { readConfig } = require('../utils');

const { initLogger, CliError } = require(path.join(__dirname, '../logger.js'));
const { errorHandler } = require(path.join(__dirname, '../../utils/utils.js'));
const { javaInstallation, javaUninstall } = require(path.join(__dirname, '../../utils/helper/java-helper.js'));
const { nginxInstallation, nginxUninstall } = require(path.join(__dirname, '../../utils/helper/nginx-helper.js'));
const { oracleInstallation, oracleUninstall } = require(path.join(__dirname, '../../utils/helper/instantclient-helper.js'));
const { rabbitmqInstallation, rabbitmqConfig, rabbitMqUninstall } = require(path.join(__dirname, '../../utils/helper/rabbitmq-helper.js'));
const { pythonInstallation, pythonUninstall } = require(path.join(__dirname, '../../utils/helper/python-helper.js'));
const { mysqlInstallation, mysqlUninstall, mysqlConfig } = require(path.join(__dirname, '../../utils/helper/mysql-helper.js'));
const { pm2Installation, pm2Uninstall } = require(path.join(__dirname, '../../utils/helper/pm2-helper.js'));

const logger = initLogger('product');

async function prerequisiteInput(all) {
  try {
    let preReq;
    let prerequisitesToInstall = { prerequisites: [] };

    if (all === 'no') {
      preReq = await prompt([
        {
          type: 'select',
          name: 'answer',
          message: 'Would you like to install any dependencies ? ',
          choices: ['yes', 'no'],
        }
      ]);
    }
    if (all === 'yes' || preReq.answer === 'yes') {
      const prerequisiteDependencies = await prompt([
        {
          type: 'select',
          name: 'all',
          message: 'Would you like to install all the dependencies ? ',
          choices: ['yes', 'no'],
        }
      ]);

      if (prerequisiteDependencies.all === 'no') {
        prerequisitesToInstall = await prompt([
          {
            type: 'multiselect',
            name: 'prerequisites',
            message: 'Select the prerequisites to install:',
            choices: ['Java', 'Oracle-InstantClient', 'RabbitMQ', 'Nginx', 'Python', 'MySQL', 'PM2'],
            initial: ['Java', 'Oracle-InstantClient', 'RabbitMQ', 'Nginx', 'Python', 'MySQL', 'PM2'],
            instructions: true,
          }
        ]);
      }
      return {
        prerequisiteDependencies,
        prerequisitesToInstall: prerequisitesToInstall.prerequisites,
        preReq,
      };
    }
    return { preReq };

  } catch (error) {
    errorHandler(error, 'product');
    throw new CliError(`Error during taking dependencies input : ${error.stack}`)
  }
}

async function prerequisite(dependencies, directory, prerequisitesToInstall) {
  try {
    const config = readConfig()

    return new Listr([
      {
        title: 'Java',
        enabled: () => {
          if (dependencies.all === 'yes') {
            return true;
          } else if (prerequisitesToInstall.includes('Java')) {
            return true;
          }
          return false;
        },
        task: async () => {
          if (config.dependencies.java.isInstalled) {
            logger.info('java is already installed');
          } else {
            return await javaInstallation(directory);
          }
        }
      },
      {
        title: 'Oracle instant-client',
        enabled: () => {
          if (dependencies.all === 'yes') {
            return true;
          } else if (prerequisitesToInstall.includes('Oracle-InstantClient')) {
            return true;
          }
          return false;
        },
        task: async () => {
          if (config.dependencies.oracleInstantClient.isInstalled) {
            logger.info('oracle instant-client is already installed');
          } else {
            return await oracleInstallation(directory);
          }
        }
      },
      {
        title: 'RabbitMQ',
        enabled: () => {
          if (dependencies.all === 'yes') {
            return true;
          } else if (prerequisitesToInstall.includes('RabbitMQ')) {
            return true;
          }
          return false;
        },
        task: async () => {
          if (config.dependencies.rabbitMq.isInstalled) {
            logger.info('rabbitmq is already installed');
          } else {
            return await rabbitmqInstallation(directory);
          }
        }
      },
      {
        title: 'NGINX',
        enabled: () => {
          if (dependencies.all === 'yes') {
            return true;
          } else if (prerequisitesToInstall.includes('Nginx')) {
            return true;
          }
          return false;
        },
        task: async () => {
          if (config.dependencies.nginx.isInstalled) {
            logger.info('nginx is already installed');
          } else {
            return await nginxInstallation(directory);
          }
        }
      },
      // {
      //   title: 'Python',
      //   enabled: () => {
      //     if (dependencies.all === 'yes') {
      //       return true;
      //     } else if (prerequisitesToInstall.includes('Python')) {
      //       return true;
      //     }
      //     return false;
      //   },
      //   task: async () => {
      //     if (config.dependencies.python.isInstalled) {
      //       logger.info('python is already installed');
      //     } else {
      //       return await pythonInstallation(directory);
      //     }
      //   }
      // },
      // {
      //   title: 'MySql',
      //   enabled: () => {
      //     if (dependencies.all === 'yes') {
      //       return true;
      //     } else if (prerequisitesToInstall.includes('MySQL')) {
      //       return true;
      //     }
      //     return false;
      //   },
      //   task: async () => {
      //     if (config.dependencies.MySql.isInstalled) {
      //       logger.info('mysql is already installed');
      //     } else {
      //       return await mysqlInstallation(directory);
      //     }
      //   }
      // },
      {
        title: 'PM2',
        enabled: () => {
          if (dependencies.all === 'yes') {
            return true;
          } else if (prerequisitesToInstall.includes('PM2')) {
            return true;
          }
          return false;
        },
        task: async () => {
          if (config.dependencies.pm2.isInstalled) {
            logger.info('pm2 is already installed');
          } else {
            return await pm2Installation(directory);
          }
        }
      }
    ]);
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during dependencies install : ${error.stack}`)
  }
}

async function dependenciesUninstall(dependenciesDetails) {
  try {
    return new Listr([
      {
        title: 'Java',
        task: async () => await javaUninstall(dependenciesDetails.java),
      },
      {
        title: 'Instantclient',
        task: async () => await oracleUninstall(dependenciesDetails.oracleInstantClient),
      },
      {
        title: 'RabbitMQ',
        task: async () => await rabbitMqUninstall(dependenciesDetails.rabbitMq),
      },
      {
        title: 'NGINX',
        task: async () => await nginxUninstall(dependenciesDetails.nginx),
      },
      {
        title: 'Python',
        task: async () => await pythonUninstall(dependenciesDetails.python),
      },
      {
        title: 'MySql',
        task: async () => await mysqlUninstall(dependenciesDetails.MySql),
      },
      {
        title: 'PM2',
        task: async () => await pm2Uninstall(dependenciesDetails.pm2),
      }
    ]);
  } catch (error) {
    logger.error(`Error occurred : ${error}`);
    errorHandler(error, 'product');
    throw new CliError(`Error occurred during Uninstall : ${error.stack}`)
  }
}

module.exports = { prerequisite, prerequisiteInput, dependenciesUninstall };
